#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <arpa/inet.h>
#include "prips.h"

/**********************************************/
/* Turn an IP address in dotted decimal into  */ 
/* a number.  This function also works  for   */
/* partial addresses.                         */
/**********************************************/
unsigned long numberize(const char *addr)
{
	int a = 0, b = 0, c = 0, d = 0;

	if(sscanf(addr, "%d.%d.%d.%d", &a, &b, &c, &d) < 1)
	{
		fprintf(stderr, "bad IP address.\n");
		exit(1);
	}
	return((a << 24) | (b << 16) | (c << 8) |  d);
}

/**********************************************/
/* Converts an IP address into dotted decimal */
/* format.                                    */
/**********************************************/
char *denumberize(unsigned long addr)
{
	static char *buffer[BUF_SIZE];

	snprintf((char *) buffer, BUF_SIZE, "%d.%d.%d.%d", 
		(addr & 0xff000000) >> 24,
		(addr & 0x00ff0000) >> 16,
		(addr & 0x0000ff00) >> 8,
		(addr & 0x000000ff));

	return((char *) buffer);
}

/**********************************************/
/* Given start and end IP addresses, we try   */
/* to find the offset used to print the range */
/* in CIDR notation.                          */
/**********************************************/
char *cidrize(unsigned long start, unsigned long end)
{
	int offset;
	static char *buffer[BUF_SIZE];

	offset = (log(end - start) / log(2)) +1;
	snprintf((char *) buffer, BUF_SIZE, "%s/%d",  
		denumberize(start), 32 - offset);

	return((char *) buffer);
}

/***********************************************/
/* Takes offset (number of bits from the left  */ 
/* of addr) and subtracts it from the number   */
/* of possible bits.  The number of possible   */
/* bits is the number of bits left for hosts.  */
/* We then return last host address.           */
/***********************************************/
unsigned long add_offset(const char *addr, int offset)
{
	if(offset > 32 || offset < 0)
	{
		fprintf(stderr, "CIDR offsets are between 0 and 32.\n");
		exit(1);
	}

	return((int)pow(2, 32 - offset) + numberize(addr) -1);
}

/************************************************/
/* Takes the address in the sockaddr_in struct  */
/* and converts it from network byte order to   */
/* a standard integer.                          */
/************************************************/ 
unsigned long denboize(struct sockaddr_in *sin)
{
        int addr, oct1, oct2, oct3, oct4;

        addr = sin->sin_addr.s_addr;
        oct1 = (addr & 0x000000ff);
        oct2 = (addr & 0x0000ff00) >> 8;
        oct3 = (addr & 0x00ff0000) >> 16;
        oct4 = (addr & 0xff000000) >> 24;

        return((oct1 << 24) | (oct2 << 16) | (oct3 << 8) | oct4);
}

unsigned long set_bits_from_right(int bits)
{
        register int i;
        unsigned long number = 0;

        for(i = 0; i < bits; i++)
                number += (int) pow(2, i);

        return number;
}

int count_on_bits(unsigned long number)
{
	unsigned long mask = 1;
	int i, on_bits = 0;

	mask <<= 31;
	for(i = 0; i < 32; i++)
	{
		if(mask & number)
			on_bits++;
		number <<= 1;
	}

	return(on_bits); 
}
 
char get_class(unsigned long address)
{
	unsigned long addr;
        char class;

        addr = (address & 0xff000000) >> 24;

        if (addr < 127)
                class = 'a';
        else if (addr >= 127 && addr < 192)
                class = 'b';
        else if (addr >= 192 && addr < 224)
                class = 'c';
        else
                class = 'd';

	return class;
}
 
